﻿using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessBL.BusinessBL
{
    public class CarBL : ICarLogic
    {
        private readonly ILogger _logger;
        private readonly ICarStorage _carStorage;

        public CarBL(ILogger<CarBL> logger, ICarStorage carStorage)
        {
            _logger = logger;
            _carStorage = carStorage;
        }
        public List<CarVM>? ReadList(CarSM? model)
        {
            _logger.LogInformation("ReadList. CarName:{CarName}.Id:{ Id}", model?.Model, model?.Id);
            var list = model == null ? _carStorage.GetFullList() : _carStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public CarVM? ReadElement(CarSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. CarName:{CarName}. Id:{ Id}", model.Model, model.Id);
            var element = _carStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(CarBM model)
        {
            CheckModel(model);
            if (_carStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(CarBM model)
        {
            CheckModel(model);
            if (_carStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(CarBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_carStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(CarBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Model))
            {
                throw new ArgumentNullException("Нет названия машины", nameof(model.Model));
            }
            if (model.Tonnage <= 0)
            {
                throw new ArgumentNullException("Грузоподъемность должна быть больше 0", nameof(model.Tonnage));
            }
            
        }
    }
}
