﻿using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class CompanyBL : ICompanyLogic
    {
        private readonly ILogger _logger;
        private readonly ICompanyStorage _companyStorage;
        public CompanyBL(ILogger<CompanyBL> logger, ICompanyStorage companyStorage)
        {
            _logger = logger;
            _companyStorage = companyStorage;
        }
        public List<CompanyVM>? ReadList(CompanySM? model)
        {
            _logger.LogInformation("ReadList. CompanyName:{CompanyName}.Id:{ Id}", model?.Title, model?.Id);
            var list = model == null ? _companyStorage.GetFullList() : _companyStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public CompanyVM? ReadElement(CompanySM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. CompanyName:{CompanyName}. Id:{ Id}", model.Title, model.Id);
            var element = _companyStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(CompanyBM model)
        {
            CheckModel(model);
            if (_companyStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(CompanyBM model)
        {
            CheckModel(model);
            if (_companyStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(CompanyBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_companyStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(CompanyBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Title))
            {
                throw new ArgumentNullException("Нет названия ", nameof(model.Title));
            }
            
            //if (model.StatusId <= 0)
            //{
            //    throw new ArgumentNullException("Стутус должн быть ", nameof(model.StatusId));
           // }
            _logger.LogInformation("Car. CarName:{CarName}.Cost:{ Cost}. Id: { Id}", model.Title);
            var element = _companyStorage.GetElement(new CompanySM
            {
                Id = model.Id,
                Title = model.Title
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Такой компания уже есть");
            }
        }
    }
}
