﻿using BusinessBL.BusinessBL;
using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class HumanBL : IHumanLogic
    {
        private readonly ILogger _logger;
        private readonly IHumanStorage _humanStorage;
        public HumanBL(ILogger<HumanBL> logger, IHumanStorage humanStorage)
        {
            _logger = logger;
            _humanStorage = humanStorage;
        }
        public List<HumanVM>? ReadList(HumanSM? model)
        {
            _logger.LogInformation("ReadList. HumanName:{HumanName}.Id:{ Id}", model?.Name, model?.Id);
            var list = model == null ? _humanStorage.GetFullList() : _humanStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public HumanVM? ReadElement(HumanSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. HumanName:{HumanName}. Id:{ Id}", model.Name, model.Id);
            var element = _humanStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(HumanBM model)
        {
            CheckModel(model);
            if (_humanStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(HumanBM model)
        {
            CheckModel(model);
            if (_humanStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(HumanBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_humanStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(HumanBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет ФИО ", nameof(model.Name));
            }
            if (string.IsNullOrEmpty(model.Phone))
            {
                throw new ArgumentNullException("Нет номера телефона", nameof(model.Phone));
            }
            
            _logger.LogInformation("Car. CarName:{CarName}.Cost:{ Cost}. Id: { Id}", model.Name);
            var element = _humanStorage.GetElement(new HumanSM
            {
                Id = model.Id,
                Phone = model.Phone
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Такой водитель уже есть");
            }
        }
    }
}
