﻿using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class PlaceBL : IPlaceLogic
    {
        private readonly ILogger _logger;
        private readonly IPlaceStorage _placeStorage;
        public PlaceBL(ILogger<PlaceBL> logger, IPlaceStorage placeStorage)
        {
            _logger = logger;
            _placeStorage = placeStorage;
        }
        
        public PlaceVM? ReadElement(PlaceSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _placeStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(PlaceBM model)
        {
            CheckModel(model);
            if (_placeStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(PlaceBM model)
        {
            CheckModel(model);
            if (_placeStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(PlaceBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_placeStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(PlaceBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Title))
            {
                throw new ArgumentNullException("Нет названия ", nameof(model.Title));
            }


            _logger.LogInformation("Car. CarName:{CarName}.Cost:{ Cost}. Id: { Id}", model.Title);
            var element = _placeStorage.GetElement(new PlaceSM
            {
                Id = model.Id,
                Title = model.Title
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Такой статус уже есть");
            }
        }

        public List<PlaceVM>? ReadList(PlaceSM model)
        {
            _logger.LogInformation("ReadList. CarName:{CarName}.Id:{ Id}", model?.Title, model?.Id);
            var list = model == null ? _placeStorage.GetFullList() : _placeStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
    }
}
