﻿using BusinessBL.BusinessBL;
using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class RouteBL : IRouteLogic
    {
        private readonly ILogger _logger;
        private readonly IRouteStorage _routeStorage;
        public RouteBL(ILogger<RouteBL> logger, IRouteStorage routeStorage)
        {
            _logger = logger;
            _routeStorage = routeStorage;
        }
        public List<RouteVM>? ReadList(RouteSM? model)
        {
            _logger.LogInformation("ReadList.  Id:{ Id}",  model?.Id);
            var list = model == null ? _routeStorage.GetFullList() : _routeStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public RouteVM? ReadElement(RouteSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _routeStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(RouteBM model)
        {
            CheckModel(model);
            if (_routeStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(RouteBM model)
        {
            CheckModel(model);
            if (_routeStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(RouteBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_routeStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(RouteBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Length <= 0)
            {
                throw new ArgumentNullException("Нет длины маршрута", nameof(model.Length));
            }
           
           
            var element = _routeStorage.GetElement(new RouteSM
            {
                Title = model.Title,
                
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Такой маршрут уже есть");
            }
        }
    }
}
