﻿using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class StatusBL : IStatusLogic
    {
        private readonly ILogger _logger;
        private readonly IStatusStorage _statusStorage;
        public StatusBL(ILogger<StatusBL> logger, IStatusStorage statusStorage)
        {
            _logger = logger;
            _statusStorage = statusStorage;
        }
        public List<StatusVM>? ReadList()
        {
            _logger.LogInformation("ReadList");
            var list = _statusStorage.GetFullList() ;
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public StatusVM? ReadElement(StatusSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _statusStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(StatusBM model)
        {
            CheckModel(model);
            if (_statusStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(StatusBM model)
        {
            CheckModel(model);
            if (_statusStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(StatusBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_statusStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(StatusBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Title))
            {
                throw new ArgumentNullException("Нет названия ", nameof(model.Title));
            }

           
            _logger.LogInformation("Car. CarName:{CarName}.Cost:{ Cost}. Id: { Id}", model.Title);
            var element = _statusStorage.GetElement(new StatusSM
            {
                Id = model.Id,
                Title = model.Title
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Такой статус уже есть");
            }
        }
    }
}
