﻿using Contracts.BindingModels;
using Contracts.BusinessLogic;
using Contracts.SearchModel;
using Contracts.Storage;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class VoyageBL : IVoyageLogic
    {
        private readonly ILogger _logger;
        private readonly IVoyageStorage _voyageStorage;
        private readonly IRouteStorage _routeStorage;
        private readonly ICompanyStorage _companyStorage;
        public VoyageBL(ILogger<VoyageBL> logger, IVoyageStorage voyageStorage,IRouteStorage routeStorage,ICompanyStorage companyStorage)
        {
            _logger = logger;
            _voyageStorage = voyageStorage;
            _routeStorage = routeStorage;
            _companyStorage = companyStorage;
        }
        public List<VoyageVM>? ReadList(VoyageSM? model)
        {
            _logger.LogInformation("ReadList.  Id:{ Id}", model?.Id);
            var list = model == null ? _voyageStorage.GetFullList() : _voyageStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public VoyageVM? ReadElement(VoyageSM model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _voyageStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(VoyageBM model)
        {
            CheckModel(model);
            if (_voyageStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        
        public bool Delete(VoyageBM model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_voyageStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(VoyageBM model, bool withParams =
       true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
           
            if (!model.DateEnd.HasValue)
            {
                throw new ArgumentNullException("нет дедлайна", nameof(model.HumanId));
            }
            if (!model.DateStart.HasValue)
            {
                throw new ArgumentNullException("нет даты выезда", nameof(model.HumanId));
            }

        }
        public List<ReportHuman> GetTopDrivers()
        {
            /*
            var list =  _voyageStorage.GetFilteredList(new VoyageSM
            {
               
                DateStart = DateTime.SpecifyKind(DateTime.Now.AddYears(-1), DateTimeKind.Utc),
                DateEnd = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc),

            });

            List<ReportHuman> reportList = new List<ReportHuman>();
            Dictionary<int? ,(string? , int? )> humans = new Dictionary<int?, (string?, int?)>();
            foreach(var item in list)
            {
                if(!humans.ContainsKey(item.HumanId))
                {
                    humans.Add(item.HumanId, (item.HumanName, item.Km));
                }else
                {
                    humans[item.HumanId] = (item.HumanName, humans[item.HumanId].Item2 + item.Km);
                }
            }
            foreach (var human in humans)
                reportList.Add(new ReportHuman { Name = human.Value.Item1, Length = human.Value.Item2 });
            return reportList;*/
            return _voyageStorage.ReportHuman();
        }
        
        public List<ReportCompany>? GetCompanysPath()
        {
            /*var list = _voyageStorage.GetFullList();
            List<ReportCompany> reportList = new List<ReportCompany>();
            Dictionary< (int? , int ?),int? > companies = new Dictionary<(int?, int?), int?>();
            foreach(var company in list)
            {
                if (!companies.ContainsKey((company.CompanyId,company.RouteId)))
                {
                    companies.Add((company.CompanyId, company.RouteId),1);
                }
                else
                {
                    
                        companies[(company.CompanyId, company.RouteId)] = (companies[(company.CompanyId, company.RouteId)].Value + 1);
                    
                }
            }
            foreach(var compan in companies)
            {

                reportList.Add(new ReportCompany { Title = _companyStorage.GetElement(new CompanySM { Id = compan.Key.Item1 }).Title, Details = _routeStorage.GetElement(new RouteSM { Id = compan.Key.Item2 }).Title, Count = compan.Value});
            }
                
            return reportList; */
            return _voyageStorage.ReportCompany();
        }
    }
}
